

CREATE TABLE IF NOT EXISTS COMMUNITY (
Type text,
Value variant default 0
);

--We need the Community table to hold some data in it
--Or else EXISTS statements will automatically return false without actually checking the query, even if other tables included do have relevant data
--See 3UC/4UC for an example scenario of this
INSERT INTO COMMUNITY
	(Type, Value)
VALUES
	('MM_GRASS', 1);

--====================
--Civ IV Leader Traits
--====================
CREATE TABLE IF NOT EXISTS 
    Leader_SharedTraits (
    LeaderType          text    REFERENCES Leaders(Type)        default null,
    TraitOne            text    REFERENCES Policies(Type)       default null,
    TraitTwo            text    REFERENCES Policies(Type)       default null);

INSERT OR REPLACE INTO Leader_SharedTraits
            (LeaderType,					TraitOne,					TraitTwo)
VALUES      ('LEADER_MM_FINN',		'POLICY_CHARISMATIC_X',		'POLICY_PROTECTIVE_X');

UPDATE Leaders 
SET Description = (CASE WHEN EXISTS(SELECT Type FROM Policies WHERE Type = 'POLICY_PHILOSOPHICAL_X' )
	THEN 'Finn [ICON_HAPPINESS_1][ICON_STRENGTH]'
	ELSE 'TXT_KEY_LEADER_MM_FINN' END) 
WHERE Type = 'LEADER_MM_FINN';


--=========
--3UC / 4UC
--=========
INSERT INTO Civilization_BuildingClassOverrides
		(CivilizationType, BuildingClassType, BuildingType)
VALUES	
		('CIVILIZATION_MM_GRASS', 'BUILDINGCLASS_3UC_DIVINEBLESSING', NULL),
		('CIVILIZATION_MM_GRASS', 'BUILDINGCLASS_MARKET', 
			(CASE WHEN EXISTS(SELECT * FROM Buildings B1, COMMUNITY C1 WHERE (B1.Type = 'BUILDING_3UC_STEEL_MILL') OR (B1.Type = 'BUILDING_AMERICA_RANCH' AND C1.Type = 'COMMUNITY_CORE_BALANCE_BUILDINGS' AND C1.Value = 1))
			THEN 'BUILDING_MM_FARMERS_MARKET'
			ELSE 'BUILDING_MARKET' END)),

		('CIVILIZATION_MM_GRASS', 'BUILDINGCLASS_BARRACKS',
			(CASE WHEN EXISTS(SELECT * FROM Buildings B1, COMMUNITY C1 WHERE (B1.Type = 'BUILDING_3UC_RANCH') OR (B1.Type = 'BUILDING_AMERICA_RANCH' AND C1.Type = 'COMMUNITY_CORE_BALANCE_BUILDINGS' AND C1.Value = 1))
			THEN 'BUILDING_MM_GAUNTLET'
			ELSE 'BUILDING_BARRACKS' END));

CREATE TRIGGER VP_MMGrasslandsUniqueComponents
AFTER INSERT ON Buildings
WHEN NEW.Type = 'BUILDING_AMERICA_RANCH' AND EXISTS(SELECT * FROM COMMUNITY WHERE Type = 'COMMUNITY_CORE_BALANCE_UNITS' AND Value = 1)
BEGIN
	INSERT INTO Civilization_BuildingClassOverrides
		(CivilizationType,			BuildingClassType,					BuildingType)
	VALUES
		('CIVILIZATION_MM_GRASS',	'BUILDINGCLASS_MARKET',				'BUILDING_MM_FARMERS_MARKET'),
		('CIVILIZATION_MM_GRASS',	'BUILDINGCLASS_BARRACKS',			'BUILDING_MM_GAUNTLET');
END;

--==========
--Vox Populi
--==========

UPDATE Buildings
SET Cost = 380
WHERE Type = 'BUILDING_MM_ADVENTURE_TREASURY' AND EXISTS (SELECT * FROM COMMUNITY WHERE Type = 'COMMUNITY_CORE_BALANCE_BUILDINGS' AND Value = 1 );

UPDATE Buildings
SET Help = 'TXT_KEY_BUILDING_MM_VP_ADVENTURE_TREASURY_HELP'
WHERE Type = 'BUILDING_MM_ADVENTURE_TREASURY' AND EXISTS (SELECT * FROM COMMUNITY WHERE Type = 'COMMUNITY_CORE_BALANCE_BUILDINGS' AND Value = 1 );

UPDATE Buildings
SET PrereqTech = 'TECH_GUILDS'
WHERE Type = 'BUILDING_MM_ADVENTURE_TREASURY' AND EXISTS (SELECT * FROM COMMUNITY WHERE Type = 'COMMUNITY_CORE_BALANCE_BUILDINGS' AND Value = 1 );

UPDATE Buildings
SET TradeRouteRecipientBonus = 0
WHERE Type = 'BUILDING_MM_ADVENTURE_TREASURY' AND EXISTS (SELECT * FROM COMMUNITY WHERE Type = 'COMMUNITY_CORE_BALANCE_BUILDINGS' AND Value = 1 );

UPDATE Buildings
SET TradeRouteTargetBonus = 0
WHERE Type = 'BUILDING_MM_ADVENTURE_TREASURY' AND EXISTS (SELECT * FROM COMMUNITY WHERE Type = 'COMMUNITY_CORE_BALANCE_BUILDINGS' AND Value = 1 );

DELETE FROM Building_YieldModifiers
WHERE BuildingType = 'BUILDING_MM_ADVENTURE_TREASURY' AND EXISTS (SELECT * FROM COMMUNITY WHERE Type = 'COMMUNITY_CORE_BALANCE_BUILDINGS' AND Value = 1 );

INSERT INTO Building_BuildingClassLocalYieldChanges
	(BuildingType,						BuildingClassType,			YieldType,			YieldChange)
VALUES
	('BUILDING_MM_ADVENTURE_TREASURY',	'BUILDINGCLASS_MINT',		'YIELD_GOLD',		(CASE WHEN EXISTS
		(SELECT * FROM COMMUNITY WHERE Type = 'COMMUNITY_CORE_BALANCE_BUILDINGS' AND Value = 1)
		THEN 3
		ELSE 0 END)),
	('BUILDING_MM_ADVENTURE_TREASURY',	'BUILDINGCLASS_CARAVANSARY','YIELD_GOLD',		(CASE WHEN EXISTS
		(SELECT * FROM COMMUNITY WHERE Type = 'COMMUNITY_CORE_BALANCE_BUILDINGS' AND Value = 1)
		THEN 3
		ELSE 0 END));

INSERT INTO Building_ResourceYieldChanges
	(BuildingType,						ResourceType,			YieldType,				Yield)
VALUES
	('BUILDING_MM_ADVENTURE_TREASURY',	'RESOURCE_GEMS',		'YIELD_GOLD',			(CASE WHEN EXISTS
		(SELECT * FROM COMMUNITY WHERE Type = 'COMMUNITY_CORE_BALANCE_BUILDINGS' AND Value = 1)
		THEN 1
		ELSE 0 END)),
	('BUILDING_MM_ADVENTURE_TREASURY',	'RESOURCE_GEMS',		'YIELD_CULTURE',		(CASE WHEN EXISTS
		(SELECT * FROM COMMUNITY WHERE Type = 'COMMUNITY_CORE_BALANCE_BUILDINGS' AND Value = 1)
		THEN 2
		ELSE 0 END)),
	('BUILDING_MM_ADVENTURE_TREASURY',	'RESOURCE_GOLD',		'YIELD_GOLD',			(CASE WHEN EXISTS
		(SELECT * FROM COMMUNITY WHERE Type = 'COMMUNITY_CORE_BALANCE_BUILDINGS' AND Value = 1)
		THEN 3
		ELSE 0 END)),
	('BUILDING_MM_ADVENTURE_TREASURY',	'RESOURCE_SILVER',		'YIELD_GOLD',			(CASE WHEN EXISTS
		(SELECT * FROM COMMUNITY WHERE Type = 'COMMUNITY_CORE_BALANCE_BUILDINGS' AND Value = 1)
		THEN 1
		ELSE 0 END)),
	('BUILDING_MM_ADVENTURE_TREASURY',	'RESOURCE_SILVER',		'YIELD_PRODUCTION',		(CASE WHEN EXISTS
		(SELECT * FROM COMMUNITY WHERE Type = 'COMMUNITY_CORE_BALANCE_BUILDINGS' AND Value = 1)
		THEN 1
		ELSE 0 END)),
	('BUILDING_MM_ADVENTURE_TREASURY',	'RESOURCE_SILVER',		'YIELD_CULTURE',		(CASE WHEN EXISTS
		(SELECT * FROM COMMUNITY WHERE Type = 'COMMUNITY_CORE_BALANCE_BUILDINGS' AND Value = 1)
		THEN 1
		ELSE 0 END));

INSERT INTO Building_YieldFromPurchase
	(BuildingType,						YieldType,				Yield)
VALUES
	('BUILDING_MM_ADVENTURE_TREASURY',	'YIELD_SCIENCE',		15);



UPDATE Buildings
SET PrereqTech = 'TECH_HORSEBACK_RIDING'
WHERE Type = 'BUILDING_MM_FARMERS_MARKET' AND EXISTS (SELECT * FROM COMMUNITY WHERE Type = 'COMMUNITY_CORE_BALANCE_BUILDINGS' AND Value = 1 );

UPDATE Buildings
SET Help = 'TXT_KEY_BUILDING_MM_VP_FARMERS_MARKET_HELP'
WHERE Type = 'BUILDING_MM_FARMERS_MARKET' AND EXISTS (SELECT * FROM COMMUNITY WHERE Type = 'COMMUNITY_CORE_BALANCE_BUILDINGS' AND Value = 1 );

UPDATE Buildings
SET WLTKDTurns = 10
WHERE Type = 'BUILDING_MM_FARMERS_MARKET' AND EXISTS (SELECT * FROM COMMUNITY WHERE Type = 'COMMUNITY_CORE_BALANCE_BUILDINGS' AND Value = 1 );

UPDATE Building_YieldChanges
SET Yield = 2
WHERE YieldType = 'YIELD_GOLD' AND BuildingType = 'BUILDING_MM_FARMERS_MARKET' AND EXISTS (SELECT * FROM COMMUNITY WHERE Type = 'COMMUNITY_CORE_BALANCE_BUILDINGS' AND Value = 1 );

UPDATE Building_YieldModifiers
SET Yield = 0
WHERE YieldType = 'YIELD_GOLD' AND BuildingType = 'BUILDING_MM_FARMERS_MARKET' AND EXISTS (SELECT * FROM COMMUNITY WHERE Type = 'COMMUNITY_CORE_BALANCE_BUILDINGS' AND Value = 1 );


UPDATE Traits
SET Description = 'TXT_KEY_TRAIT_MM_VP_ADVENTURE_TIME'
WHERE Type = 'TRAIT_MM_ADVENTURE_TIME' AND EXISTS (SELECT * FROM COMMUNITY WHERE Type = 'COMMUNITY_CORE_BALANCE_CIVS' AND Value = 1 );

UPDATE Buildings
SET Defense = 2000
WHERE Type = 'BUILDING_MM_CS_DEFENSE' AND EXISTS (SELECT * FROM COMMUNITY WHERE Type = 'COMMUNITY_CORE_BALANCE_CIVS' AND Value = 1 );

UPDATE Buildings
SET Defense = 200
WHERE Type = 'BUILDING_MM_GRASS_DEFENSE' AND EXISTS (SELECT * FROM COMMUNITY WHERE Type = 'COMMUNITY_CORE_BALANCE_CIVS' AND Value = 1 );

UPDATE Building_YieldChanges
SET Yield = 2
WHERE YieldType = 'YIELD_GOLD' AND BuildingType = 'BUILDING_MM_GRASS_DEFENSE' AND EXISTS (SELECT * FROM COMMUNITY WHERE Type = 'COMMUNITY_CORE_BALANCE_CIVS' AND Value = 1 );

INSERT INTO Building_YieldChanges
	(BuildingType,					YieldType,		Yield)
SELECT
	'BUILDING_MM_GRASS_DEFENSE',	'YIELD_FOOD',	2
WHERE EXISTS (SELECT * FROM COMMUNITY WHERE Type = 'COMMUNITY_CORE_BALANCE_CIVS' AND Value = 1 );

UPDATE Buildings
SET CitySupplyFlat = 2
WHERE Type = 'BUILDING_MM_CS_GOLD' AND EXISTS (SELECT * FROM COMMUNITY WHERE Type = 'COMMUNITY_CORE_BALANCE_BUILDINGS' AND Value = 1 );



UPDATE Buildings
SET Help = 'TXT_KEY_BUILDING_MM_VP_GAUNTLET_HELP'
WHERE Type ='BUILDING_MM_GAUNTLET' AND EXISTS (SELECT * FROM COMMUNITY WHERE Type = 'COMMUNITY_CORE_BALANCE_BUILDINGS' AND Value = 1 );

UPDATE Buildings
SET CitySupplyFlat = 1
WHERE Type = 'BUILDING_MM_GAUNTLET' AND EXISTS (SELECT * FROM COMMUNITY WHERE Type = 'COMMUNITY_CORE_BALANCE_BUILDINGS' AND Value = 1 );

UPDATE Buildings
SET CitySupplyFlat = 1
WHERE Type = 'BUILDING_MM_GAUNTLET_RUINS' AND EXISTS (SELECT * FROM COMMUNITY WHERE Type = 'COMMUNITY_CORE_BALANCE_BUILDINGS' AND Value = 1 );

INSERT INTO Building_UnhappinessNeedsFlatReduction
	(BuildingType, YieldType, Yield)
VALUES
	('BUILDING_MM_GAUNTLET', 'YIELD_PRODUCTION', 1);



UPDATE Units
SET Combat = 7
WHERE Type = 'UNIT_MM_ADVENTURER' AND EXISTS (SELECT * FROM COMMUNITY WHERE Type = 'COMMUNITY_CORE_BALANCE_UNITS' AND Value = 1 );

UPDATE UnitPromotions
SET WorkRateMod = 50
WHERE Type = 'PROMOTION_MM_ADVENT' AND EXISTS (SELECT * FROM COMMUNITY WHERE Type = 'COMMUNITY_CORE_BALANCE_UNITS' AND Value = 1 );

--UPDATE: Found a nifty new way to do these insert statements!
INSERT INTO Unit_FreePromotions
	(UnitType,				PromotionType)
SELECT						
    'UNIT_MM_ADVENTURER',	'PROMOTION_BARBARIAN_BOUNS'
WHERE EXISTS (SELECT * FROM COMMUNITY WHERE Type = 'COMMUNITY_CORE_BALANCE_UNITS' AND Value = 1 );




INSERT INTO Improvement_YieldAdjacentTwoSameType
	(ImprovementType, YieldType, Yield)
VALUES
	('IMPROVEMENT_FARM_MM_ADV', 'YIELD_FOOD', 1);

INSERT INTO Building_ImprovementYieldChanges
		(BuildingType,	ImprovementType,				YieldType,	Yield)
SELECT	 BuildingType,	('IMPROVEMENT_FARM_MM_ADV'),	YieldType,	Yield
FROM Building_ImprovementYieldChanges WHERE ImprovementType = 'IMPROVEMENT_FARM';

INSERT INTO Building_ImprovementYieldChanges
		(BuildingType,	ImprovementType,				YieldType,	Yield)
SELECT	 BuildingType,	('IMPROVEMENT_MINE_MM_ADV'),	YieldType,	Yield
FROM Building_ImprovementYieldChanges WHERE ImprovementType = 'IMPROVEMENT_MINE';

INSERT INTO Building_ImprovementYieldChanges
		(BuildingType,	ImprovementType,				YieldType,	Yield)
SELECT	 BuildingType,	('IMPROVEMENT_CAMP_MM_ADV'),	YieldType,	Yield
FROM Building_ImprovementYieldChanges WHERE ImprovementType = 'IMPROVEMENT_CAMP';





