-- 
-- Author: Typhlomenace
-- DateCreated: 6/20/2016 12:57:19 AM
-- Taken from Typhlomenace's K-On mod 
-- It allows Ness to say the player's name, but it has the ability to do more if I ever to decide to do more -MM
--------------------------------------------------------------

-- Based on the unique dialog code used for Vice Virtuoso's Neptune civilization

gLeader = "LEADER_MM_FINN"
gLeaderTextKey = "TXT_KEY_LEADER_MM_FINN"

-- ============================================================================
-- Enables debug statements
-- ============================================================================
bMMDebug = true

function dprint(...)
  if (bMMDebug) then
    --print(string.format(...))
	  print(...)
  end
end

-- ============================================================================
-- Locals
-- ============================================================================
-- References to table entries


-- ============================================================================
-- SetAndResizeLeaderHeadDialogBox
-- ============================================================================
-- From VV - changes the size of the text box for diplomacy
function SetAndResizeLeaderHeadDialogBox(sString)
	if sString then
		--LeaderHeadRoot
		local speechBox = ContextPtr:LookUpControl("/LeaderHeadRoot/LeaderSpeech")
		if speechBox then
			speechBox:SetText(sString)
			local border = ContextPtr:LookUpControl("/LeaderHeadRoot/LeaderSpeechBorderFrame")
			local frame = ContextPtr:LookUpControl("/LeaderHeadRoot/LeaderSpeechFrame")
			if border and frame then
			local contentSize = speechBox:GetSizeY() + 48;
				border:SetSizeY( contentSize );
				frame:SetSizeY( contentSize - 4 );
			end
		end
		--DiscussionDialog
		local speechBox2 = ContextPtr:LookUpControl("/LeaderHeadRoot/DiscussionDialog/LeaderSpeech")
		if speechBox2 then
			speechBox2:SetText(sString)
			local border2 = ContextPtr:LookUpControl("/LeaderHeadRoot/DiscussionDialog/LeaderSpeechBorderFrame")
			local frame2 = ContextPtr:LookUpControl("/LeaderHeadRoot/DiscussionDialog/LeaderSpeechFrame")
			if border2 and frame2 then
			local contentSize = speechBox:GetSizeY() + 48;
				border2:SetSizeY( contentSize );
				frame2:SetSizeY( contentSize - 4 );
			end
		end
		--DiploTrade
		local speechBox2 = ContextPtr:LookUpControl("/LeaderHeadRoot/DiploTrade/DiscussionText")
		if speechBox2 then
			speechBox2:SetText(sString)
			local border2 = ContextPtr:LookUpControl("/LeaderHeadRoot/DiploTrade/LeaderSpeechBorderFrame")
			local frame2 = ContextPtr:LookUpControl("/LeaderHeadRoot/DiploTrade/LeaderSpeechFrame")
			if border2 and frame2 then
			local contentSize = speechBox:GetSizeY() + 48;
				border2:SetSizeY( contentSize );
				frame2:SetSizeY( contentSize - 4 );
			end
		end
	end
end


--Localized Lua string functions
local gsub = string.gsub
local reverse = string.reverse
local sub = string.sub
local find = string.find

-- ============================================================================
-- SubstituteEndNumberForPercentSign
-- ============================================================================
-- From VV - Replaces the number at the end of a text tag to a percentage sign
function SubstituteEndNumberForPercentSign(sString)
--	dprint("String before substitution: "..sString)
	local sReverse = reverse(sString)
	local iStop = find(sReverse, "_")
	if not iStop then return sString end
	sReverse = sub(sReverse, iStop + 1)
	sString = reverse(sReverse).."%"
--	dprint("String after substitution: "..sString)
	return sString
end

--Find all dialogue from Ness, store it as a key to the row's tag to be retrieved later
local tNessResponses = {}
local tExactTags = {}										   
for row in DB.Query("SELECT * FROM Language_en_US WHERE Tag LIKE ('" .. gLeaderTextKey .. "_%')") do
	local sString = row.Text
--	dprint(row.Tag)
--	dprint(row.Text)
	if row.Text then 
		if find(sString, "##MM##") then
--			dprint("This is a message that needs its key changed!")
			local s = sub(row.Text, 6, string.len(row.Text))
			sString = sub(row.Text, 0, find(s, "##") + 4) 
--			dprint(sString)
		end
		tNessResponses[sString] = SubstituteEndNumberForPercentSign(row.Tag)
		tExactTags[sString] = sub(row.Tag, string.len("TXT_KEY_" .. gLeader .. "_"))
	end
end

-- ============================================================================
-- GetRandomDiploString
-- ============================================================================
-- From VV - Picks a random text string from the given tag (made into a
-- wildcard by SubstituteEndNumberForPercentSign)
function GetRandomDiploString(sString)
	local locale = "Language_" ..Locale.GetCurrentLanguage().Type
	local t = {}
	for row in DB.Query("SELECT * FROM "..locale.." WHERE Tag LIKE ('"..sString.."')") do
		t[#t + 1] = row.Text
	end
	if #t > 0 then
		return t[Game.Rand(#t, "Random Text Roll") + 1]
	end
	return
end
-- ============================================================================
-- Typh_DiploStringInsertPlayerName
-- ============================================================================
-- Replaces "INSERT_PLAYER_NAME_HERE" with the active player's leader's name
function Typh_DiploStringInsertPlayerName(sString, iActivePlayerType)
	local sActivePlayerName
	sActivePlayerName = Locale.ConvertTextKey(GameInfo.Leaders[iActivePlayerType].Description)
	local sActivePlayerFirstName
	for sWord in string.gmatch(sActivePlayerName, "%S+") do
		sActivePlayerFirstName = sWord
		break
	end
	sString = gsub(sString, "INSERT_PLAYER_NAME_HERE", sActivePlayerFirstName)
	--Marius, here!
	--This portion allows Lucas to st-st-stutter the player's name
	--How it works:
		--In the corresponding localization file, I will insert the text '##I##1' before 'INSERT_PLAYER_NAME_HERE'
		--The ##I## tells LucasResponses.lua that we want to stutter the player's name here
		--And the number tells the script how much of the original name to keep in the stutter, starting at the first letter
			--We will just need to be careful, since there are currently no checks for how long the original name is
	if find(sString, "##I##") then
		local sReplace = sub(sString, find(sString, "##I##"), find(sString, "##I##") + 5)
		local iStutterTo = tonumber(sub(sReplace, 6, 6))
		local sStutter = sub(sActivePlayerFirstName, 0, iStutterTo)
		sString = gsub(sString, sReplace, sStutter)
	end
	return sString
end


--For handling messages by approachType
	local tApproaches = {
		[MajorCivApproachTypes.MAJOR_CIV_APPROACH_HOSTILE] = "HOSTILE",
		[MajorCivApproachTypes.MAJOR_CIV_APPROACH_GUARDED] = "HOSTILE",
		[MajorCivApproachTypes.MAJOR_CIV_APPROACH_FRIENDLY] = "FRIENDLY",
		[MajorCivApproachTypes.MAJOR_CIV_APPROACH_AFRAID] = "AFRAID",
		[MajorCivApproachTypes.MAJOR_CIV_APPROACH_NEUTRAL] = "NEUTRAL"
--		[MajorCivApproachTypes.MAJOR_CIV_APPROACH_WAR] = "AT_WAR" --GetApproachTowardsUsGuess never returns this!
	}
	--For handling messages talking about a specific person
	local tLeaders = {
		["Ness"] = "ABOUT_NESS",
		["Eagleland"] = "ABOUT_NESS",
		["Eagle"] = "ABOUT_NESS",

		["Ninten"] = "ABOUT_NINTEN",
		--Here's where this gets difficult - we have no way of discerning NUSA from other Americas
		--["The United States"] = "ABOUT_NINTEN",

		["Porky"] = "ABOUT_PORKY",
		["The Pigmasks"] = "ABOUT_PORKY",
		["Pigmask"] = "ABOUT_PORKY",
		["The Porky Empire"] = "ABOUT_PORKY",

		["Bowser"] = "ABOUT_BOWSER",
		["The Koopa Troop"] = "ABOUT_BOWSER",
		["Koopa Troop"] = "ABOUT_BOWSER"
		}


local sPreviousString = ""
--Used for previous leader names, civ names, etc
local sData = ""

function GetRelevantLeader(sString, sTag)
	local sLeader = tLeaders[sString]
	local sNewTag = sub(sTag, find(sTag, gLeaderTextKey)) .. "_" .. (sLeader)
	sNewTag = sNewTag .. sub(sTag, string.len(gLeaderTextKey) + 1) 
	return sNewTag
end
function GetFollowupResponse(szLeaderMessage, sTag)
	local sString;

	dprint("Previous Message with Lucas: " .. sPreviousString)
	if sTag then
		local sOldTag = tExactTags[sPreviousString]
		if sOldTag then
			--First, see if there's a specific followup response for this message in this context
			local sNewTag = gLeaderTextKey .. "_FLW" .. sOldTag .. (sub(sTag, string.len(gLeaderTextKey .. "_")) or "%")
			dprint("Checking for response... " .. sNewTag)
			sString = GetRandomDiploString(sNewTag)
			--If not, then see if there's a generic followup response for this message in this context
			if not sString then
				sOldTag = sub(sOldTag, 0, string.len(sOldTag) - 2)
			--	sOldTag = sub(sOldTag, string.len(gLeaderTextKey .. "_", string.len(sOldTag) - 3))
				sNewTag = gLeaderTextKey .. "_FLW" .. sOldTag .. (sub(sTag, string.len(gLeaderTextKey .. "_")) or "%")
				dprint("Checking for response... " .. sNewTag)
				szLeaderMessage = GetRandomDiploString(sNewTag) or szLeaderMessage
			else szLeaderMessage = sString
			end
		end
	end
	return szLeaderMessage
end

function FixStringData(szLeaderMessage)
local sMessage;  
	-- if find(szLeaderMessage, "##MM##") then
		dprint(szLeaderMessage)
		--Split up the message, get the tag
			--Cut off ##MM##
			szLeaderMessage = sub(szLeaderMessage, 7, string.len(szLeaderMessage))
			--Split the string
			sMessage = sub(szLeaderMessage, 0, find(szLeaderMessage, "##") - 1)
			dprint(sMessage)
			sTag = tNessResponses["##MM##" .. sMessage]


			--Not sure on the logic needed to implement here, we'll have to see
			--szLeaderMessage = GetFollowupResponse(szLeaderMessage)

			sPreviousString = "##MM##" .. sMessage
			local sInsert = sub(szLeaderMessage, find(szLeaderMessage, "##"), string.len(szLeaderMessage))
			dprint(sInsert)
			
			--Start processing the second half of the string... do replacements where necessary
			--Leader Name--
			if find(sInsert, "##PLAY##") then
				local sPart = sub(sInsert, find(sInsert, "##PLAY##") + 8)
				--We may need to cut the string further, check for pound signs
				dprint(sPart)
				if sPart then
					if find(sPart, "##") then sPart = sub(sPart, 0, find(sPart, "##") - 1) end
					sMessage = gsub(sMessage, "INSERT_LEADER_NAME_HERE", sPart)
					--This line might be replaceable with one specific to this leader/civ
					if tLeaders[sPart] then
						local sNewTag = GetRelevantLeader(sPart, sTag)
						sString = GetRandomDiploString(sNewTag)
						if sString then sTag = sNewTag
							sMessage = sString
							--We're not using this line anymore, no need to continue
							sInsert = "We're done here."
						end
					end
					sData = sPart
				end
			end
			--Civilization Name--
			if find(sInsert, "##CIVN##") then
				local sPart = sub(sInsert, find(sInsert, "##CIVN##") + 8)
				dprint(sPart)
				if sPart then
					if find(sPart, "##") then sPart = sub(sPart, 0, find(sPart, "##") - 1) end
					sMessage = gsub(sMessage, "INSERT_CIV_NAME_HERE", sPart)
					if tLeaders[sPart] then
						local sNewTag = GetRelevantLeader(sPart, sTag)
						sString = GetRandomDiploString(sNewTag)
						if sString then sTag = sNewTag
							sMessage = sString
							sInsert = "We're done here."
						end
					end
					sData = sPart
				end
			end
			--Civilization Adjective--
			if find(sInsert, "##ADJT##") then
				local sPart = sub(sInsert, find(sInsert, "##ADJT##") + 8)
				dprint(sPart)
				if sPart then
					if find(sPart, "##") then sPart = sub(sPart, 0, find(sPart, "##") - 1) end
					sMessage = gsub(sMessage, "INSERT_CIV_ADJECTIVE_HERE", sPart)
					if tLeaders[sPart] then
						local sNewTag = GetRelevantLeader(sPart, sTag)
						sString = GetRandomDiploString(sNewTag)
						if sString then sTag = sNewTag
							sMessage = sString
							sInsert = "We're done here."
						end
					end
					sData = sPart
				end
			end
			--League Name--
			if find(sInsert, "##LEAG##") then
				local sPart = sub(sInsert, find(sInsert, "##LEAG##") + 8)
				dprint(sPart)
				if sPart then
					if find(sPart, "##") then sPart = sub(sPart, 0, find(sPart, "##") - 1) end
					sData = sPart
					sMessage = gsub(sMessage, "INSERT_LEAGUE_NAME_HERE", sPart)
				end
			end
			--City Name
			if find(sInsert, "##CITY##") then
				local sPart = sub(sInsert, find(sInsert, "##CITY##") + 8)
				dprint(sPart)
				if sPart then
					if find(sPart, "##") then sPart = sub(sPart, 0, find(sPart, "##") - 1) end
					sData = sPart
					sMessage = gsub(sMessage, "INSERT_CITY_NAME_HERE", sPart)
				end
			end
	szLeaderMessage = sMessage
	--end
	return szLeaderMessage, sTag
end

-- ============================================================================
-- DialogOverride
-- ============================================================================
-- From VV - Used to override the current diplomacy line with a unique one if
-- it's available
 
function NessDialogOverride(iPlayer, iDiploUIState, szLeaderMessage, iAnimation, iData)
	if GameInfo.Leaders[Players[iPlayer]:GetLeaderType()].Type ~= gLeader then return end
	
	local sString;
	local startTime = os.clock()
	local function DoOverride(tableType)
		local iActivePlayer = Game:GetActivePlayer()
		local pActivePlayer = Players[iActivePlayer]
		local sActivePlayerType = GameInfo.Leaders[pActivePlayer:GetLeaderType()].Type
		dprint("Active Player's Type is "..sActivePlayerType)

		--local sNewTag = gsub(tableType[szLeaderMessage], "LEADER_TYPH_", "UD_VS_"..gsub(sActivePlayerType, "LEADER_", "").."_")

		--dprint("Tag created is "..sNewTag)
		--sString = GetRandomDiploString(sNewTag)
		if not sString then
			sString = szLeaderMessage -- for now, this is fine, but we may want to use GetRandomDiploString in the future
		end
		
		-- This replaces "INSERT_PLAYER_NAME_HERE" with the player's name
		if find(sString, "INSERT_STRING_DATA_HERE") then
			sString = gsub(sString, "INSERT_STRING_DATA_HERE", sData)
		end
		if (find(sString, "INSERT_PLAYER_NAME_HERE")) then
			dprint("Replacing 'INSERT_PLAYER_NAME_HERE' with the player's name...")
			local iActivePlayer = Game:GetActivePlayer()
			local pActivePlayer = Players[iActivePlayer]
			local sActivePlayerType = GameInfo.Leaders[pActivePlayer:GetLeaderType()].Type
			sString = Typh_DiploStringInsertPlayerName(sString, pActivePlayer:GetLeaderType())
			dprint("Message: "..sString)
		else
			dprint("The current line doesn't contain 'INSERT_PLAYER_NAME_HERE' - we don't need to insert the player's name.")
		end
	end

	local sTag = tNessResponses[szLeaderMessage]
	szLeaderMessage = GetFollowupResponse(szLeaderMessage, sTag)
	
	local bFixedData = false
	if find(szLeaderMessage, "##MM##") then
		bFixedData = true
		szLeaderMessage, sTag = FixStringData(szLeaderMessage)
		sPreviousString = "##MMM##" .. szLeaderMessage
	end

	
	dprint(sTag)
	--See if we can find a tag based on ApproachType
	local pTalkingPlayer = Players[iPlayer]
	local pTalkingTeam = Teams[pTalkingPlayer:GetTeam()]
	local iActivePlayer = Game.GetActivePlayer()
	local pActivePlayer = Players[iActivePlayer]

	local sApproach = tApproaches[pActivePlayer:GetApproachTowardsUsGuess(iPlayer)] or ""
	local sNewTag = sub(sTag, find(sTag, gLeaderTextKey .. "_")) .. (sApproach)
	sNewTag = sNewTag .. sub(sTag, string.len(sNewTag) - string.len(sApproach))

	if pTalkingTeam:IsAtWar(pActivePlayer:GetTeam()) then
		sNewTag = gsub(sNewTag, sApproach, "AT_WAR")
	end

	dprint(sNewTag)

	if sNewTag ~= sTag then
		sString = GetRandomDiploString(sNewTag)
	end
	dprint("Old Message: " .. szLeaderMessage)
	dprint("New Message: " .. (sString or ""))

	
	if not bFixedData then
		sPreviousString = sString or szLeaderMessage
	end

	if sTag then
		DoOverride(tNessResponses)	
	end

	

	if sString then
		SetAndResizeLeaderHeadDialogBox(sString)
	end
	
	dprint("Total Time to accomplish unique dialog: "..os.clock() - startTime)
end
Events.AILeaderMessage.Add(NessDialogOverride)


