-- MMGrassFunctions
-- Author: Marius
-- DateCreated: 8/23/2019 9:29:00 PM
--------------------------------------------------------------
include "MMGrassGlobals.lua"
-----------------------------------------------
--OnPlayersMeet
--Fires whenever two teams meet
--If one player is the Grasslands and the other a city-state
--Award the Grasslands with a Food and Gold bonus
-----------------------------------------------
function OnGrassMeetMinor(iActivePlayer, iPlayerMet)
	--See if one player is the Grasslands and another is a city-state
	if not((IsGrasslands(iActivePlayer) or IsGrasslands(iPlayerMet)) and (Players[iActivePlayer]:IsMinorCiv() or Players[iPlayerMet]:IsMinorCiv())) then return end
	local pPlayer;
	local pMinor;
	if IsGrasslands(iActivePlayer) then
		pPlayer = Players[iActivePlayer]
		pMinor = Players[iPlayerMet]
	else
		pPlayer = Players[iPlayerMet]
		pMinor = Players[iActivePlayer]
	end
	print("The Grasslands has met a city state")
	local pMinorTeam = Teams[pMinor:GetTeam()]

	local iGold = 90
	if gCBP then
		iGold = 120
	end
	
	--See if this the first player the city-state has met
	for k, v in pairs(Teams) do
		if pMinorTeam:IsHasMet(k) and not(k == pPlayer:GetTeam() or k == pMinor:GetTeam()) then --IsHasMet returns true if it's yourself
			print("Someone else has met this city state all ready")
			iGold = iGold / 3
			break
		elseif not Players[Teams[k]:GetLeaderID()] then --Is this a player?
		elseif Players[Teams[k]:GetLeaderID()]:IsMinorCiv() then break  
		end
	end
	pPlayer:ChangeGold(iGold)
	pPlayer:GetCapitalCity():ChangeFood(iGold / 3)
	if gCBP then
		pMinor:ChangeMinorCivFriendshipWithMajor(pPlayer:GetID(), iGold / 3)
	end
	pPlayer:AddNotification(NotificationTypes.NOTIFICATION_GENERIC, "Math! +" .. iGold .. " " .. Locale.ConvertTextKey("[ICON_GOLD]") .. " Gold and " .. iGold / 3 .. " " .. Locale.ConvertTextKey("[ICON_FOOD]") .. " Food in your capital from meeting a City-State.", "City-State met!", pMinor:GetCapitalCity():GetX(), pMinor:GetCapitalCity():GetY())
end
GameEvents.TeamMeet.Add(OnGrassMeetMinor)


----------------------------------------------------
--GrassTechDiscovered
--When the Grasslands discovers Mining, Trapping, or even Agriculture, make sure they also get the associated dummy tech
--that will enable Adventurers to construct the dummy improvements
----------------------------------------------------
function GrassTechDiscovered(iTeam, iTech, bAdopted)
	if not bAdopted then return end
	if not (iTech == gAgriculture or iTech == gMining or iTech == gTrapping) then return end
	--Teams has no built in method to iterate through their members
	--We have to iterate through all alive major civs and find our Grasslands civ and verify they belong to this team
	--But see if we can find the potential Grasslands with a call to GetLeaderID first
	local pTeam = Teams[iTeam]
	local iPlayer = pTeam:GetLeaderID()
	if not IsGrasslands(iPlayer) then
		local bGrass = false
		for i = 0, GameDefines.MAX_MAJOR_CIVS - 1 do
			local pPlayer = Players[i]
			if pPlayer:IsEverAlive() and pPlayer:IsAlive() and IsGrasslands(iPlayer) then
				bGrass = true
				iPlayer = i
			end
		end
	--	if not bGrass then return end
	end
	if iTech == gAgriculture then
		--Give the player the dummy tech
		--But hide the fact that the player has received an extra technology from them
		pTeam:GetTeamTechs():SetHasTech(gAgriculture, false, iPlayer, false, false)
		pTeam:GetTeamTechs():SetHasTech(GameInfoTypes.TECH_FARM_MM_ADV, true, iPlayer, false, false)
		pTeam:GetTeamTechs():SetHasTech(gAgriculture, true, iPlayer, false, true)
	elseif iTech == gMining then
		pTeam:GetTeamTechs():SetHasTech(gMining, false, iPlayer, false, false)
		pTeam:GetTeamTechs():SetHasTech(GameInfoTypes.TECH_MINE_MM_ADV, true, iPlayer, false, false)
		pTeam:GetTeamTechs():SetHasTech(gMining, true, iPlayer, false, true) 
	else
		pTeam:GetTeamTechs():SetHasTech(gTrapping, false, iPlayer, false, false)
		pTeam:GetTeamTechs():SetHasTech(GameInfoTypes.TECH_CAMP_MM_ADV, true, iPlayer, false, false)
		pTeam:GetTeamTechs():SetHasTech(gTrapping, true, iPlayer, false, true)
	end
end
GameEvents.TeamSetHasTech.Add(GrassTechDiscovered)

------------------------------------------------------
--AdventurerLevelUp
--Increases an Adventurers Combat Strength by 2 on each level up
------------------------------------------------------
function AdventurerLevelUp(iPlayer, iUnit, ePromotion)
	if not IsGrasslands(iPlayer) then return end
	local pPlayer = Players[iPlayer]
	local pUnit = pPlayer:GetUnitByID(iUnit)
	if not g_tAdventurer[pUnit:GetUnitType()] then return end
	pUnit:SetBaseCombatStrength(GameInfo.Units[pUnit:GetUnitType()].Combat + ((pUnit:GetLevel() - 1) * 3))
end
GameEvents.UnitPromoted.Add(AdventurerLevelUp)

function OnGrassDoTurn(iPlayer)
	if not IsGrasslands(iPlayer) then return end
	dprint("OnGrassDoTurn")
	local iProtectingMinors = GetNumProtectingMinors(iPlayer)
	local pPlayer = Players[iPlayer]
	g_tTreasuries[iPlayer] = 0

	pPlayer:GetCapitalCity():SetNumRealBuilding(gGrassGold, iProtectingMinors)
	for pCity in pPlayer:Cities() do
		if pCity:IsHasBuilding(gAdventureTreasury) then
			g_tTreasuries[iPlayer] = g_tTreasuries[iPlayer] + 1
		end
		pCity:SetNumRealBuilding(gGrassDefense, iProtectingMinors)
		pCity:SetNumRealBuilding(gFarmersBonus, 0)
	end
	FarmersMarketBonus(pPlayer:GetTradeRoutes())
--	end
end
GameEvents.PlayerDoTurn.Add(OnGrassDoTurn)

function OnAdventureTreasuryBuilt(iPlayer, iCity, eBuilding)
	OnGauntletBuilt(iPlayer, iCity, eBuilding)
	if not(eBuilding == gAdventureTreasury) then return end
	local pPlayer = Players[iPlayer]
	local pCity = pPlayer:GetCityByID(iCity)
	local iGold = 150
	pPlayer:ChangeGold(iGold)
	pPlayer:AddNotification(NotificationTypes.NOTIFICATION_GENERIC, iGold .. " " .. Locale.ConvertTextKey("[ICON_GOLD]") .. " Gold from constructing an Adventurer's Treasury in " .. pCity:GetName(), "Adventurer's Treasury!")
end
GameEvents.CityConstructed.Add(OnAdventureTreasuryBuilt)

local iPreviousCity = -1
function ApplyTreasuryDiscount(iPlayer, iCity, eBuilding)

	local pCapital = Players[iPlayer]:GetCapitalCity()
	if not (iCity == iPreviousCity) then
		pCapital:SetNumRealBuilding(gTreasuryGold, 0)
		iPreviousCity = iCity
	end
	

	local pCity = Players[iPlayer]:GetCityByID(iCity)
	if pCity:IsHasBuilding(gAdventureTreasury) and not(pCapital:IsHasBuilding(gTreasuryGold)) then
		pCapital:SetNumRealBuilding(gTreasuryGold, 1)
	end

	if eBuilding == gGauntletSalvage and pCity:IsHasBuilding(gGauntletRuins) then
		return true
	elseif eBuilding == gGauntletSalvage then
		return false
	elseif eBuilding == gGauntlet and pCity:IsHasBuilding(gGauntletRuins) then
		return false
	else
		return true
	end
end
GameEvents.CityCanConstruct.Add(ApplyTreasuryDiscount)


function AdventurerTrained(iPlayer, iCity, iUnit, bGold, bFaithOrCulture)
	if not IsGrasslands(iPlayer) then return end
	local pPlayer = Players[iPlayer]
	local pCity = pPlayer:GetCityByID(iCity)
	if pCity:IsHasBuilding(gGauntlet) then
		local pUnit = pPlayer:GetUnitByID(iUnit)
		if pUnit:IsCombatUnit() then
			if g_tAdventurer[pUnit:GetUnitType()] then
				pUnit:SetHasPromotion(gAdventurerPromo, true)
				pUnit:ChangeExperience(8)
				DetermineGauntletStatus(pCity)
			else
				pUnit:SetHasPromotion(gGauntletPromo, true)
				DetermineGauntletStatus(pCity)
			end
		end
	end
end
GameEvents.CityTrained.Add(AdventurerTrained)


local tImprovements = {
	[gFarm] = true,
	[gMine] = true,
	[gCamp] = true
}
function AdventurerImprovementComplete(iPlayer, x, y, eImprovement)
	if not tImprovements[eImprovement] then return end
	local pPlot = Map.GetPlot(x, y)
	local pPlayer = Players[iPlayer]
	for i = 0, pPlot:GetNumUnits() - 1 do
		local pUnit = pPlot:GetUnit(i)
		if g_tAdventurer[pUnit:GetUnitType()] and pUnit:GetOwner() == iPlayer then
			dprint("Rewarding xp")
			local xp = 3
			if pUnit:IsHasPromotion(gAdventurerPromo) then
				xp = 5
			end
			pUnit:ChangeExperience(xp)

			pPlayer:ChangeGold(2 * g_tTreasuries[iPlayer])
		end
		if pUnit:GetUnitType() == GameInfoTypes.UNIT_MM_ADVENTURER_TREASURY then
			pPlayer:ChangeGold(20)
		end
	end
	dprint(pPlot:GetOwner())
	if pPlot:GetOwner() > -1 then
		local pPlotOwner = Players[pPlot:GetOwner()]
		if pPlotOwner:IsMinorCiv() then
			dprint("Awarding Player " .. iPlayer .. " with influence")
			pPlotOwner:ChangeMinorCivFriendshipWithMajor(iPlayer, 20)
		end
	else
		dprint("Adventurer improvement built in neutral territory")
		DetermineExtraImprovementBonus(iPlayer, x, y, eImprovement)
	end
end
GameEvents.BuildFinished.Add(AdventurerImprovementComplete)

function OnBarbarianKilled(iOwner, iKilledUnit, iUnitType, x, y, bDelay, iKillerOwner)
	if not(iOwner == 63) then return end
	if not IsGrasslands(iKillerOwner) then return end
	local pPlayer = Players[iKillerOwner]
	local pUnit = Players[iOwner]:GetUnitByID(iKilledUnit)
	if pUnit:IsHasPromotion(gMineBarbarian) then
		local iGold = math.floor(pUnit:GetBaseCombatStrength() * 5)
		pPlayer:ChangeGold(iGold)
		pPlayer:AddNotification(NotificationTypes.NOTIFICATION_GENERIC, "+" .. iGold .. " " .. Locale.ConvertTextKey("[ICON_GOLD]") .. " Gold from killing a dungeon monster", "Dungeon creature slain!", x, y)
	elseif pUnit:IsHasPromotion(gCampBarbarian) then
		local pNewUnit = pPlayer:InitUnit(iUnitType, x, y, UNITAI_ATTACK)
--		pNewUnit:Convert(pUnit)
		pNewUnit:JumpToNearestValidPlot()
		pNewUnit:FinishMoves()
		pPlayer:AddNotification(NotificationTypes.NOTIFICATION_GENERIC, "The hostile barbarian concedes! Impressed by the power of your Adventurer, they agree to serve under your command!", "Do thy splits!", x, y)
	end
end
GameEvents.UnitPrekill.Add(OnBarbarianKilled)
	